package com.pointrlabs.sample.map

import android.content.Context
import androidx.appcompat.app.AppCompatActivity
import com.pointrlabs.core.map.models.events_listeners.MapEventsListener
import com.pointrlabs.core.management.models.Building
import android.os.Bundle
import com.pointrlabs.sample.R
import com.pointrlabs.core.map.viewmodels.PTRMapWidgetConfiguration
import com.pointrlabs.core.map.views.PTRMapWidgetFragment
import com.pointrlabs.core.management.Pointr
import android.content.Intent
import android.os.Build
import android.widget.TextView
import androidx.activity.result.IntentSenderRequest
import androidx.activity.result.contract.ActivityResultContracts
import com.pointrlabs.core.management.models.Site
import com.pointrlabs.core.map.handlers.ExitButtonEventsHandler
import com.pointrlabs.core.map.views.PTRMapFragment
import com.pointrlabs.sample.utils.handleLocationRequest

class MapActivity : AppCompatActivity(), MapEventsListener {


    private lateinit var building: Building

    private val pointr
        get() = Pointr.getPointr()

    private val locationServiceRequest =
        registerForActivityResult(ActivityResultContracts.StartIntentSenderForResult()) {
            val permissionManager = pointr?.permissionManager ?: return@registerForActivityResult
            // Enable bluetooth services
            permissionManager.requestBluetoothService()
        }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_map_widget)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        val siteManager = pointr?.siteManager ?: return

        val buildingInternalIdentifier = intent.getIntExtra(BUNDLE_BUILDING_ID, 0)
        // if you want to delegate activity recreation handling to PointrSDK
        // get the widget instance after calling show method
        val configuration = PTRMapWidgetConfiguration.defaultConfiguration()
        PTRMapWidgetFragment.newInstance(configuration) { widgetFragment ->
            widgetFragment.show(supportFragmentManager, R.id.fragment_container)

            building = siteManager.getBuilding(buildingInternalIdentifier) ?: run {
                val sites = siteManager.getSites()
                if (sites.isEmpty()) return@newInstance
                // Pick a random site as given building cannot be found
                val site = sites.last()
                val buildings = site.buildings
                if (buildings.isEmpty()) return@newInstance
                // Pick a random building from within the site as given building cannot be found
                buildings.last()
            }

            widgetFragment.apply {
                showBuilding(this@MapActivity.building)
                addListener(this@MapActivity)
                exitButtonEventsHandler = object : ExitButtonEventsHandler() {
                    override fun onExitClicked(
                        mapWidget: PTRMapWidgetFragment,
                        exitButton: TextView
                    ) {
                        super.onExitClicked(mapWidget, exitButton)
                        finish()
                    }
                }
            }

            this.title = building.title
            handlePermissionInOrder(REQUEST_CODE_INITIAL_STATE)
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        handlePermissionInOrder(requestCode)
    }

    private fun handlePermissionInOrder(requestCode: Int) {
        pointr?.permissionManager?.apply {
            when {
                !hasLocationPermissionWhileInUse && requestCode == REQUEST_CODE_INITIAL_STATE -> requestLocationPermissionForWhileInUse(
                    this@MapActivity,
                    REQUEST_CODE_LOCATION_PERMISSION
                )

                !hasBluetoothPermission && requestCode <= REQUEST_CODE_LOCATION_PERMISSION && Build.VERSION.SDK_INT >= Build.VERSION_CODES.S -> requestBluetoothPermission(
                    this@MapActivity,
                    REQUEST_CODE_BLUETOOTH_PERMISSION
                )

                !isLocationServicesEnabled -> handleLocationRequest { resolutionApiException ->
                    resolutionApiException?.let {
                        locationServiceRequest.launch(
                            IntentSenderRequest.Builder(it.resolution).build()
                        )
                    } ?: run {
                        requestBluetoothService()
                    }
                }

                !isBluetoothServicesEnabled -> requestBluetoothService()
            }
        }
    }


    override fun mapDidDetectFocusedSite(
        mapFragment: PTRMapFragment,
        site: Site?,
        buildings: List<Building>
    ) {
        if (buildings.isEmpty()) {
            title = ""
            return
        }
        if (buildings.contains(building)) return
        building = buildings.first()
        title = building.title
    }


    companion object {
        private const val BUNDLE_BUILDING_ID = "BUNDLE_BUILDING_ID"

        const val REQUEST_CODE_INITIAL_STATE = 0
        const val REQUEST_CODE_LOCATION_PERMISSION = 1
        const val REQUEST_CODE_BLUETOOTH_PERMISSION = 2

        /**
         * Creates an intent to start this activity
         *
         * @param buildingId to load and display the [PTRMapWidgetFragment]
         * @return intent to start []
         */
        @JvmStatic
        fun newIntent(
            context: Context,
            buildingId: Int
        ): Intent {
            val intent = Intent(context, MapActivity::class.java)
            intent.putExtra(BUNDLE_BUILDING_ID, buildingId)
            return intent
        }
    }

    override fun onSupportNavigateUp(): Boolean {
        onBackPressedDispatcher.onBackPressed()
        return super.onSupportNavigateUp()
    }
}